Summary: The Linux kernel (the core of the Linux operating system)

# What parts do we want to build?  We must build at least one kernel.
# These are the kernels that are built IF the architecture allows it.

%define build_yhpc 0 
%define build_madwifi 0
%define build_at76c503 0
%define buildup 1
%define buildsmp 0
%define buildsource 0
%define usegcc32 0
%define build_pseries 0
%define build_maple 0
%define build_cell 0 
# Versions of various parts

#
# Polite request for people who spin their own kernel rpms:
# please modify the "release" field in a way that identifies
# that the kernel isn't the stock distribution kernel, for example by
# adding some text to the end of the version number.
#
%define sublevel 16
%define bkrev 1
%define rpmrev 1
%define kversion 2.6.%{sublevel}
%define kversionfull 2.6.%{sublevel}-%{bkrev}
%define rpmversion 2.6.%{sublevel}
%if %{build_yhpc}
%define release %{bkrev}.yhpc.%{rpmrev}
%else
%define release %{bkrev}.ydl.%{rpmrev}
%endif

%define KVERREL %{PACKAGE_VERSION}-%{PACKAGE_RELEASE}

%define at76c503_name at76c503a
%define at76c503_ver 20051201

# groups of related archs
%define all_x86 i386 i586 i686
%define all_sparc sparc sparc64
%define all_ppc ppc ppc64 ppciseries ppcpseries

 #Override generic defaults with per-arch defaults 

# First, architecture-specific kernels off on all other archs (ifnarch)
%ifarch i686 i586
%define buildsource 0
%endif

# Second, per-architecture exclusions (ifarch)
%ifarch i386
%define buildsmp 0
%define buildup 0
%endif

%ifarch %{all_x86}
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-i?86*.config
%define _64bit 0
%endif

%ifarch x86_64
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-x86_64*.config
%define _64bit 1
%endif

%ifarch ppc64pseries ppc64iseries
%define buildsmp 0
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-ppc64*series.config
%define _64bit 1
%endif

%ifarch s390
%define buildsmp 0
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-s390.config
%endif

%ifarch s390x
%define buildsmp 0
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-s390x.config
%endif

%ifarch sparc
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-sparc*.config
%define _64bit 0
# SMP is hosed for sparc in 2.6 right now...
%define buildsmp 0
# We need to build the sparc kernel with gcc32
%define usegcc32 1
%endif

%ifarch sparc64
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-sparc64*.config
%define _64bit 1
%endif

%ifarch ppc
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-ppc*.config
%define _64bit 0
%endif

%ifarch ppc64
%define all_arch_configs $RPM_SOURCE_DIR/kernel-%{kversion}-ppc64*.config
%define _64bit 1
%endif

# Three sets of minimum package version requirements in the form of Conflicts:
# to versions below the minimum
#

#
# First the general kernel 2.6 required versions as per
# Documentation/Changes
#
%define kernel_dot_org_conflicts  ppp <= 2.3.15, pcmcia-cs <= 3.1.20, isdn4k-utils <= 3.0, mount < 2.10r-5, nfs-utils < 1.0.3, e2fsprogs < 1.29, util-linux < 2.10, jfsutils < 1.0.14, reiserfsprogs < 3.6.3, xfsprogs < 2.1.0, procps < 2.0.9, oprofile < 0.5.3

# 
# Then a series of requirements that are distribution specific, either 
# because we add patches for something, or the older versions have 
# problems with the newer kernel or lack certain things that make 
# integration in the distro harder than needed.
#
%define package_conflicts  cipe < 1.4.5, tux < 2.1.0, kudzu <= 0.92, initscripts < 7.23, dev < 3.2-7, iptables < 1.2.5-3, bcm5820 < 1.81, nvidia-rh72 <= 1.0

#
# Several packages had bugs in them that became obvious when the NPTL
# threading code got integrated. 
#
%define nptl_conflicts SysVinit < 2.84-13, pam < 0.75-48, vixie-cron < 3.0.1-73, privoxy < 3.0.0-8, spamassassin < 2.44-4.8.x,  cups < 1.1.17-13

#
# Packages that need to be installed before the kernel is, because the %post
# scripts use them.
#
%define kernel_prereq  fileutils, module-init-tools, initscripts >= 5.83, mkinitrd >= 4.2.11

Name: kernel
Group: System Environment/Kernel
License: GPLv2
Version: %{rpmversion}
Release: %{release}
ExclusiveArch: %{all_x86} x86_64 s390 s390x %{all_sparc} %{all_ppc}
ExclusiveOS: Linux
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# We can't let RPM do the dependencies automatic because it'll then pick up
# a correct but undesirable perl dependency from the module headers which
# isn't required for the kernel proper to function
AutoReqProv: no

#
# List the packages used during the kernel build
#
BuildPreReq: module-init-tools, patch >= 2.5.4, bash >= 2.03, sh-utils, tar
BuildPreReq: bzip2, findutils, dev, gzip, m4, perl, make >= 3.78, gnupg
BuildConflicts: rhbuildsys(DiskFree) < 500Mb
BuildRequires: binutils >= 2.12, yellowdog-rpm-config
%if %{usegcc32}
BuildRequires: gcc32
%else
BuildRequires: gcc >= 2.96-98
%endif
%ifarch ppc64
Obsoletes: kernel-g5
%endif


Source0: ftp://ftp.kernel.org/pub/linux/kernel/v2.5/linux-%{kversionfull}.tar.bz2

%if %{build_yhpc}
Source1: infiniband.tar.bz2
%endif
Source2: mptlinux-3.01.14.23-src.tar.gz

%if %{build_yhpc}
Source3: logo_yhpc_clut224.ppm
%else
Source3: logo_ydl_clut224.ppm
%endif

# at76c503 wireless driver
Source4: %{at76c503_name}-%{at76c503_ver}.tar.gz
Source5: madwifi.tgz
Source10: COPYING.modules
Source11: linux-modules-unsupported

Source20: kernel-%{version}-ppc.config
Source21: kernel-%{version}-ppc-smp.config
Source24: kernel-%{version}-ppc64-g5.config
Source25: kernel-%{version}-ppc64-g5-smp.config
Source26: kernel-%{version}-ppc64-pseries.config
Source27: kernel-%{version}-ppc64-maple.config
Source28: kernel-%{version}-ppc64-cell.config
#
# Patches 0 through 100 are meant for core subsystem upgrades
# Patches 100 through 500 are meant for architecture patches
# 200 - 300   x86(-64)

# 300 - 350   ppc/ppc64
Patch300: linux-2.6-softmac-git.patch
Patch301: linux-2.6-bcm43xx-git.patch
Patch302: linux-2.6-bcm43xx-neuter.patch

Patch330: patch_01-15_SPU_file_system_2.6.14.patch
Patch331: patch_02-15_switchable_spu_contexts_for_spufs_2.6.14.patch
Patch332: patch_03-15_add_kernel-side_context_switch_code_for_spufs_2.6.14.patch
Patch333: patch_04-15_add_spu-side_context_switch_code_for_spufs_2.6.14.patch
Patch334: patch_05-15_schedule_support_for_spufs_2.6.14.patch
Patch335: patch_06-15_enable_pause_zero_2.6.14.patch
Patch336: patch_07-15_hacks_to_run_BPA_hw_2.6.14.patch
Patch337: patch_08-15_spufs_and_sparsemem_extreme_2.6.14.patch
Patch338: patch_09-15_support_for_old_cell_hw_2.6.14.patch
Patch339: patch_10-15_add_RTAS_console_driver_2.6.14.patch
Patch340: patch_11-15_support_IBMfullsystemsimulator_2.6.14.patch
Patch341: patch_12-15_default_config_updates_2.6.14.patch
Patch342: patch_13-15_IBMfullsystemsimulator_consoledriver_2.6.14.patch
Patch343: patch_14-15_Fix_oops_when_spufs_module_not_loaded_2.6.14.patch
Patch344: patch_15-15_turnoff_spufs_debugging_2.6.14.patch


Patch400: linux-2.6.12-at76c503a.patch
# 350 - 400   s390(x)

#
# 400 - sparc(64)
#

#
# Patches 500 through 1000 are reserved for bugfixes to the core system
# and patches related to how RPMs are build
#

Patch500: linux-2.6-build-nonintconfig.patch
# FINALLY MADE IT UPSTREAM
# http://linux.bkbits.net:8080/linux-2.5/cset@1.1608.98.42
#Patch580: linux-2.6.1-loop-changefd.patch

#
# Patches 1000 to 5000 are reserved for bugfixes to drivers and filesystems
#


#
# External drivers that are about to get accepted upstream
#

#
# 10000 to 20000 is for stuff that has to come last due to the
# amount of drivers they touch. But only theese should go here. 
# Not patches you're too lazy for to put in the proper place.
#

# END OF PATCH DEFINITIONS

BuildRoot: %{_tmppath}/kernel-%{KVERREL}-root

%description 
The kernel package contains the Linux kernel (vmlinux), the core of your
Yellow Dog Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.
%ifarch ppc64
%package g5
Summary: The Linux kernel compiled for SMP machines.

Group: System Environment/Kernel
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
Obsoletes: kernel-enterprise < 2.4.10


%description g5
The kernel package contains the Linux kernel (vmlinux), the core of your
Yellow Dog Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.
%endif

%package source
Summary: The source code for the Linux kernel.
Group: Development/System
Prereq: fileutils
Requires: make >= 3.78
%if %{usegcc32}
Requires: gcc32
%else
Requires: gcc >= 3.2
%endif
Requires: /usr/bin/strip
# for xconfig and gconfig
#Requires: qt-devel, gtk2-devel readline-devel

%description source
The kernel-source package contains the source code files for the Linux
kernel. The source files can be used to build a custom kernel that is
smaller due only including drivers for your particular hardware, if you are
so inclined (and you know what you're doing). The customisation guide in the
documentation describes in detail how to do this. This package is neither
needed nor usable for building external kernel modules for linking into the
default kernel.

%package doc
Summary: Various documentation bits found in the kernel source.
Group: Documentation

%description doc
This package contains documentation files from the kernel
source. Various bits of information about the Linux kernel and the
device drivers shipped with it are documented in these files. 

You'll want to install this package if you need a reference to the
options that can be passed to Linux kernel modules at load time.

%ifarch ppc
%package smp
Summary: The Linux kernel compiled for SMP machines.

Group: System Environment/Kernel
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
Obsoletes: kernel-enterprise < 2.4.10

%description smp
This package includes a SMP version of the Linux kernel. It is
required only on machines with two or more CPUs as well as machines with
hyperthreading technology.

Install the kernel-smp package if your machine uses two or more CPUs.


%else
%package g5-smp
Summary: The Linux kernel compiled for SMP machines.

Group: System Environment/Kernel
Provides: kernel-smp = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
Obsoletes: kernel-enterprise < 2.4.10

%description g5-smp
This package includes a SMP version of the Linux kernel. It is
required only on machines with two or more CPUs as well as machines with
hyperthreading technology.

Install the kernel-smp package if your machine uses two or more CPUs.


%endif

%package pseries
Summary: The Linux kernel compiled for IBM JS20 Blade Servers
Group: System Environment/Kernel
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
Obsoletes: kernel-enterprise < 2.4.10

%description pseries
This package includes a JS20 version of the Linux kernel. It is
required only on IBM JS20 Blade Servers

%package maple
Summary: The Linux kernel compiled for the Momentum Maple evaluation platform 
Group: System Environment/Kernel
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
Obsoletes: kernel-enterprise < 2.4.10

%description maple
This package includes a Maple version of the Linux kernel. It is
required only on the Momentum Maple 970FX evaluation platform.

%package cell
Summary: The Linux kernel compiled for the Mercury CELL evaluation platform 
Group: System Environment/Kernel
Provides: kernel = %{version}
Provides: kernel-drm = 4.3.0
Prereq: %{kernel_prereq}
Conflicts: %{kernel_dot_org_conflicts}
Conflicts: %{package_conflicts}
Conflicts: %{nptl_conflicts}
# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
Obsoletes: kernel-enterprise < 2.4.10

%description cell
This package includes a CELL version of the Linux kernel. It is
required only on the Mercury CELL evaluation platform.

%prep

%setup -q -n %{name}-%{version} -c
cd linux-%{kversionfull}

#
# Patches 0 through 100 are meant for core subsystem upgrades
# 

#
# Architecture patches
#
#
# x86(-64)
#

#
# Clean up stack=8Kb assumptions and switch to 4Kb stacks
#

#
# Add irqstacks
#

# 
# ppc64
%ifarch ppc ppc64
%patch300 -p1
%patch301 -p1
%patch302 -p1

%endif

%ifarch ppc
%endif 

%ifarch ppc64

%if %{build_cell}
%patch330 -p1
%patch331 -p1
%patch332 -p1
%patch333 -p1
%patch334 -p1
%patch335 -p1
%patch336 -p1
%patch337 -p1
%patch338 -p1
%patch339 -p1
%patch340 -p1
%patch341 -p1
%patch342 -p1
%patch343 -p1
%patch344 -p1


%endif
%endif

cp %{SOURCE3} drivers/video/logo/logo_linux_clut224.ppm

# Patch for Kconfig and Makefile build issues

# Export symbols needed by some modules


#
# s390
#

# Basic build fixes

#
# sparc/sparc64
#
#
# Patches 500 through 1000 are reserved for bugfixes to the core system
# and patches related to how RPMs are build
#
# This patch adds a "make nonint_oldconfig" which is non-interactive and
# also gives a list of missing options at the end. Useful for automated
# builds (as used in the buildsystem).
%patch500 -p1
#%patch501 -p1
%if %{build_yhpc}
#%patch502 -p1
%endif

#
# Patch that adds a __must_check attribute for functions for which checking
# the return value is mantadory (eg copy_from_user)


#
# Exec shield
#

#
# 4G/4G split
#

#
# Add make buildcheck target
#

#
# loop change FD ioctl; needed for anaconda to allow
# the cd on which anaconda resided to be ejected
#

# FINALLY MADE IT UPSTREAM:
# http://linux.bkbits.net:8080/linux-2.5/cset@1.1608.98.42
#%patch580 -p1


#
# TUX
#

#
# Patches 1000 to 5000 are reserved for bugfixes to drivers and filesystems
#


# add vidfail capability; 
# without this patch specifying a framebuffer on the kernel prompt would
# make the boot stop if there's no supported framebuffer device; this is bad
# for the installer cd that wants to automatically fall back to textmode
# in that case
# XXX: breaks ppc and ppc64 builds for now

#
# Rik Faith's auditing patch
#

#
# and vsyscalls for amd64
#

#
# Fix a bug in selinx that breaks dhcp
#

#
# Make /dev/mem a need-to-know function 
#

#
# Most^WAll users of sleep_on are broken; fix a bunch
#

#
# Fix ACPI sleep for 4g/4g split
#

#
# Patches 5000 to 6000 are reserved for new drivers that are about to
# be merged upstream
#


#
# final stuff
#

#
# misc small stuff to make things compile or otherwise improve performance
#

#
# Add MODULE_LICENSE() where it's missing
#
# %patch10010 -p1


# END OF PATCH APPLICATIONS
%if %{build_at76c503}
tar zxvf %{SOURCE4}
%patch400 -p0
%endif

%if %{build_madwifi}
tar zxvf %{SOURCE5}
MADWIFI_PATH=`pwd`
pushd .
cd madwifi/patches/2.6
bash install.sh $MADWIFI_PATH
popd
%endif

# if we're using gcc32, lets tell the Makefiles to do that.
%if %{usegcc32}
perl -p -i -e "s/= gcc/= gcc32/" Makefile
perl -p -i -e "s/\)gcc/\)gcc32/" Makefile
%endif


cp %{SOURCE10} Documentation/

mkdir configs
cp -fv %{all_arch_configs} .

# sparc makes this a little trickier, we don't want to try
# to parse sparc64 config files, so they get removed here first.
%ifarch sparc
rm -rf kernel-%{kversion}-sparc64*.config
%endif
%ifarch ppc
rm -rf kernel-%{kversion}-ppc64*.config
%endif

# now run oldconfig over all the config files
for i in *.config
do
	mv $i .config
	make nonint_oldconfig > /dev/null
	mv .config configs/$i
done

# make sure the kernel has the sublevel we know it has. This looks weird
# but for -pre and -rc versions we need it since we only want to use
# the higher version when the final kernel is released.
perl -p -i -e "s/^SUBLEVEL.*/SUBLEVEL = %{sublevel}/" Makefile
perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -prep/" Makefile

# get rid of unwanted files resulting from patch fuzz
find . -name "*.orig" -exec rm -fv {} \;
find . -name "*~" -exec rm -fv {} \;

###
### build
###
%build

cd linux-%{kversionfull}

BuildKernel() {
    export TARGET=powerpc-be-eabi
    export KERNELRELEASE=$KVERREL


    # Pick the right config file for the kernel we're building
    if [ -n "$1" ] ; then
	Config=kernel-%{kversion}-%{_target_cpu}-$1.config
    else
	Config=kernel-%{kversion}-%{_target_cpu}.config
    fi

    KernelVer=%{version}-%{release}$1
    echo BUILDING A KERNEL FOR $1 %{_target_cpu}...

    # make sure EXTRAVERSION says what we want it to say
    perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -%{release}$1/" Makefile


    # and now to start the build process
    make -s mrproper
    cp configs/$Config .config

    make -s nonint_oldconfig > /dev/null
    make -s include/linux/version.h 

%ifarch %{all_sparc}
    make -s %{?_smp_mflags} image
%endif
%ifarch %{all_ppc}
    make -s %{?_smp_mflags}
%endif
%if %{_target_cpu} == "maple"
    make zImage
%endif
%if %{_target_cpu} == "pseries"
    make zImage
%endif
if [ "$1" = "cell" ]; then
    make zImage
fi

%ifnarch %{all_sparc} %{all_ppc}
    make -s %{?_smp_mflags} bzImage 
%endif
    make -s %{?_smp_mflags} modules || exit 1
#    make buildcheck
    
    # Start installing the results

    mkdir -p $RPM_BUILD_ROOT/usr/lib/debug/boot
    install -m 644 System.map $RPM_BUILD_ROOT/usr/lib/debug/boot/System.map-$KernelVer
%if %{_64bit}
    cat System.map  | grep "r __ksymtab" | cut -c30- > exported
%else
    cat System.map  | grep "r __ksymtab" | cut -c22- > exported
%endif
	for i in `cat exported`
	do
		grep "$i\$" System.map >> $RPM_BUILD_ROOT/boot/System.map-$KernelVer || :
		grep "tab_$i\$" System.map >> $RPM_BUILD_ROOT/boot/System.map-$KernelVer || :
		grep "__crc_$i\$" System.map >> $RPM_BUILD_ROOT/boot/System.map-$KernelVer || :
	done
    rm -f exported
    install -m 644 .config $RPM_BUILD_ROOT/boot/config-$KernelVer
    install -m 644 System.map $RPM_BUILD_ROOT/boot/System.map-$KernelVer
%ifarch %{all_sparc}
    cp arch/*/boot/image $RPM_BUILD_ROOT/boot/vmlinux-$KernelVer
%endif
%ifarch %{all_ppc}
    cp vmlinux $RPM_BUILD_ROOT/boot/vmlinux-$KernelVer
%endif
%ifarch ppc64
    if [ -n "$1" ] ; then
       if [ "$1" = "maple" ]; then
          if [ -f arch/ppc64/boot/zImage ]; then
             cp arch/ppc64/boot/zImage $RPM_BUILD_ROOT/boot/zImage-$KernelVer
          fi
       fi
       if [ "$1" = "pseries" ]; then
          if [ -f arch/ppc64/boot/zImage ]; then
             cp arch/ppc64/boot/zImage $RPM_BUILD_ROOT/boot/zImage-$KernelVer
          fi
       fi
       if [ "$1" = "cell" ]; then
          if [ -f arch/ppc64/boot/zImage ]; then
             cp arch/ppc64/boot/zImage $RPM_BUILD_ROOT/boot/zImage-$KernelVer
          fi
       fi

    fi
%endif
%ifnarch %{all_sparc} %{all_ppc}
    cp arch/*/boot/bzImage $RPM_BUILD_ROOT/boot/vmlinuz-$KernelVer
%endif

    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer
    make -s INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install KERNELRELEASE=$KernelVer
    # at76c503 wireless driver
%if %{build_at76c503}
    cd %{at76c503_name}-%{at76c503_ver}
    make KSRC=../ KVERS=$KernelVer
    make KSRC=../ KVERS=$KernelVer  DESTDIR=$RPM_BUILD_ROOT install
    cd ..
%endif

    # And save the headers/makefiles etc for building modules against
    #
    # This all looks scary, but the end result is supposed to be:
    # * all arch relevant include/ files
    # * all Makefile/Kconfig files
    # * all script/ files 

    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # first copy everything
    cp --parents `find  -type f -name Makefile -o -name "Kconfig*"` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build 
    # then drop all but the needed Makefiles/Kconfig files
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Documentation
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cp arch/%{_arch}/kernel/asm-offsets.s $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch}/kernel || :
    cp .config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp -a scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*.o
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*/*.o
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cd include
    cp -a acpi config linux math-emu media net pcmcia rxrpc scsi sound video asm asm-m68k asm-generic asm-powerpc $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cp -a `readlink asm` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    # Make sure the Makefile and version.h have a matching timestamp so that
    # external modules can be built
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Makefile $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/version.h
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/autoconf.h
    cd .. 

    #
    # save the vmlinux file for kernel debugging into the kernel-debuginfo rpm
    #
    mkdir -p $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/$KernelVer
    cp vmlinux $RPM_BUILD_ROOT/usr/lib/debug/lib/modules/$KernelVer

    # move the modules in the SOURCE11 file to the unsupported/ directory so
    # that they end up in the -unsupported-modules rpms
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/unsupported
    pushd $RPM_BUILD_ROOT/lib/modules/$KernelVer/kernel ; {
	 tar cf - `cat %{SOURCE11}`  | tar xf - -C ../unsupported
	 rm -rf `cat %{SOURCE11}` 
    } ; popd


    # mark modules executable so that strip-to-file can strip them
    find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko" -type f  | xargs chmod u+x

    # detect missing or incorrect license tags
    # for i in `find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko" ` ; do echo -n "$i " ; /sbin/modinfo -l $i >> modinfo ; done
    # cat modinfo | grep -v "GPL" | grep -v "Dual BSD/GPL" | grep -v "Dual MPL/GPL" | grep -v "GPL and additional rights" | grep -v "GPL v2" && exit 1 
    # rm -f modinfo
    # remove files that will be auto generated by depmod at rpm -i time
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.*

}

###
# DO it...
###

# prepare directories
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/boot

%if %{buildup}
%ifarch ppc64
BuildKernel g5
%else
BuildKernel
%endif
%endif

%if %{buildsmp}
%ifarch ppc64
BuildKernel g5-smp
%else
BuildKernel smp
%endif
%endif

%if %{build_pseries}
BuildKernel pseries
%endif

%if %{build_maple}
BuildKernel maple
%endif

%if %{build_cell}
BuildKernel cell
%endif

###
### install
###

%install

cd linux-%{kversionfull}

# architectures that don't get kernel-source (i586/i686/athlon) dont need
# much of an install because the build phase already copied the needed files

%if %{buildsource}

mkdir -p $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}

# clean up the source tree so that it is ready for users to build their own
# kernel
make -s mrproper

# copy the source over
tar cf - . | tar xf - -C $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}

# set the EXTRAVERSION to <version>custom, so that people who follow a kernel building howto
# don't accidentally overwrite their currently working moduleset and hose
# their system
perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -%{release}custom/" $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}/Makefile
install -m 644 %{SOURCE10}  $RPM_BUILD_ROOT/usr/src/linux-%{KVERREL}
%endif

chmod 644 $RPM_BUILD_ROOT/lib/modules/*/build/scripts/kconfig/conf
###
### clean
###

%clean
rm -rf $RPM_BUILD_ROOT

###
### scripts
###

# load the loop module for upgrades...in case the old modules get removed we have
# loopback in the kernel so that mkinitrd will work.

%ifarch ppc
%pre 
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0

%pre smp
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0
%else
%pre g5
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0

%pre g5-smp
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0
%endif

%if %{build_pseries}
%pre pseries
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0
%endif

%if %{build_maple}
%pre maple
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0
%endif

%if %{build_maple}
%pre cell
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
exit 0
%endif


%ifarch ppc
%post 
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :

%post smp
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}smp
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :
%else 
%post g5
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}g5
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :
%post g5-smp
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}g5-smp
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :

%endif

%if %{build_pseries}
%post pseries
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}pseries
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :
%endif

%if %{build_cell}
%post cell 
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}cell
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :
%endif


%if %{build_maple}
%post maple
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --mkinitrd --depmod --install %{KVERREL}maple
[ -x /usr/sbin/hardlink ] && /usr/sbin/hardlink -c /lib/modules/2.6.* || :
%endif

%ifarch ppc
%preun 
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}

%preun smp
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}smp
%else
%preun g5
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}g5

%preun g5-smp
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}g5-smp
%endif

%if %{build_pseries}
%preun pseries
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}pseries
%endif

%if %{build_maple}
%preun maple
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}maple
%endif

%if %{build_cell}
%preun cell
/sbin/modprobe loop 2> /dev/null > /dev/null  || :
[ -x /sbin/new-kernel-pkg ] && /sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}cell
%endif


###
### file lists
###

%ifarch ppc
%if %{buildup}
%files 
%defattr(-,root,root)
/boot/vmlinu?-%{KVERREL}
/boot/System.map-%{KVERREL}
/boot/config-%{KVERREL}
%dir /lib/modules/%{KVERREL}
/lib/modules/%{KVERREL}/kernel
%verify(not mtime) /lib/modules/%{KVERREL}/build
/lib/modules/%{KVERREL}/unsupported
%endif

%if %{buildsmp}
%files smp
%defattr(-,root,root)
/boot/vmlinu?-%{KVERREL}smp
/boot/System.map-%{KVERREL}smp
/boot/config-%{KVERREL}smp
%dir /lib/modules/%{KVERREL}smp
/lib/modules/%{KVERREL}smp/kernel
%verify(not mtime) /lib/modules/%{KVERREL}smp/build
/lib/modules/%{KVERREL}smp/unsupported
%endif

%else
%if %{buildup}
%files g5
%defattr(-,root,root)
/boot/vmlinu?-%{KVERREL}g5
/boot/System.map-%{KVERREL}g5
/boot/config-%{KVERREL}g5
%dir /lib/modules/%{KVERREL}g5
/lib/modules/%{KVERREL}g5/kernel
%verify(not mtime) /lib/modules/%{KVERREL}g5/build
/lib/modules/%{KVERREL}g5/unsupported
%endif

%if %{buildsmp}
%files g5-smp
%defattr(-,root,root)
/boot/vmlinu?-%{KVERREL}g5-smp
/boot/System.map-%{KVERREL}g5-smp
/boot/config-%{KVERREL}g5-smp
%dir /lib/modules/%{KVERREL}g5-smp
/lib/modules/%{KVERREL}g5-smp/kernel
%verify(not mtime) /lib/modules/%{KVERREL}g5-smp/build
/lib/modules/%{KVERREL}g5-smp/unsupported
%endif
%endif

%if %{build_pseries}
%files pseries
%defattr(-,root,root)
/boot/zImage-%{KVERREL}maple
/boot/vmlinu?-%{KVERREL}pseries
/boot/System.map-%{KVERREL}pseries
/boot/config-%{KVERREL}pseries
%dir /lib/modules/%{KVERREL}pseries
/lib/modules/%{KVERREL}pseries/kernel
%verify(not mtime) /lib/modules/%{KVERREL}pseries/build
/lib/modules/%{KVERREL}pseries/unsupported
%endif

%if %{build_maple}
%files maple
%defattr(-,root,root)
/boot/zImage-%{KVERREL}maple
/boot/vmlinu?-%{KVERREL}maple
/boot/System.map-%{KVERREL}maple
/boot/config-%{KVERREL}maple
%dir /lib/modules/%{KVERREL}maple
/lib/modules/%{KVERREL}maple/kernel
%verify(not mtime) /lib/modules/%{KVERREL}maple/build
/lib/modules/%{KVERREL}maple/unsupported
%endif

%if %{build_cell}
%files cell 
%defattr(-,root,root)
/boot/zImage-%{KVERREL}cell
/boot/vmlinu?-%{KVERREL}cell
/boot/System.map-%{KVERREL}cell
/boot/config-%{KVERREL}cell
%dir /lib/modules/%{KVERREL}cell
/lib/modules/%{KVERREL}cell/kernel
%verify(not mtime) /lib/modules/%{KVERREL}cell/build
/lib/modules/%{KVERREL}cell/unsupported
%endif


# only some architecture builds need kernel-source and kernel-doc

%if %{buildsource}

%files source
%defattr(-,root,root)
/usr/src/linux-%{KVERREL}/

%files doc
%defattr(-,root,root)
%doc linux-%{kversionfull}/Documentation/*

%endif

%changelog
* Tue Mar 21 2006 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- update to 2.6.16

* Mon Mar 20 2006 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- try 2.6.16rc6
- add patch for bcm43xx
- add patches so 2.6.16rc6 compiles (this should be fixed with proper solutions)

* Thu Nov 10 2005 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- bump to 2.6.14
- add support for CELL
- add lots of patches for 2.6.14 for better imacg5/therm support
- remove at76c503a wifi stuff

* Wed Sep 21 2005 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- merge Dan's at76c503a wifi stuff including:
- add at76c503a wireless driver
- patch at76c503a makefile
 
* Tue Aug 31 2005 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- add the pci radeon patch
- enable xfs quota support

* Wed Aug 24 2005 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- enable LVM
- bump mkinitrd req ver to 4.2.11

* Fri Aug 19 2005 Dan Burcaw <dan@ydl.net>
- do not require build_yhpc for pseries/maple

* Tue Jul 19 2005 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- version bump to 2.6.12

* Thu Apr 14 2005 Dan Burcaw <dan@ydl.net>
- added kernel patch for e1000 confirguration

* Tue Apr 05 2005 Dan Burcaw <dan@ydl.net>
- changed maple pci_probe_only to 1
- changed NLS defaults for USB flash drive support

* Fri Apr 01 2005 Dan Burcaw <dan@ydl.net>
- enabled AMD8111 ethernet support on Maple

* Thu Mar 24 2005 Dan Burcaw <dan@ydl.net>
- add an apache patch

* Fri Mar 18 2005 Dan Burcaw <dan@ydl.net>
- compile ZLIB_DEFLATE on Maple

* Sun Feb 20 2005 Dan Burcaw <dan@ydl.net>
- added build_maple & build_pseries
- added stub/initrd bits

* Thu Feb 10 2005 Dan Burcaw <dan@ydl.net>
- added maple_setup patch

* Wed Feb 09 2005 Dan Burcaw <dan@ydl.net>
- added build_yhpc
- remove -g5 sub-packages (32-bit g5 kernel is obsolete)
- rename js20 -> pseries pkg

* Sun Feb 06 2005 Dan Burcaw <dan@ydl.net>
- 2.6.10 merge; removed unneeded patches
- sync'd with YDL 4.0.1 spec
- added maple package & patch

* Wed Dec 01 2004 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- add thermal and oldworld patches

* Sat Nov 13  2004 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- update configs for RAID and Fusion MPT

* Tue Oct 26 2004 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- merge 2.6.9, remove old unneeded patches
- add fpu bug fix and thermal support for new G5s
- make a js20 kernel package

* Thu Sep 15 2004 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- add a couple patches from benh

* Fri Aug 20 2004 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- merge 2.6.8
- merge  PowerMac7,3 fixes

* Thu Jun 17 2004 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- merge 2.6.7
- remove patches that have been merged upstream

* Wed May 19 2004 Owen Stampflee <ostampflee@terrasoftsolutions.com>
- merge 2.6.6 final
- support the new iBook G4 video chipset
- preliminary support for XServe G5s

* Mon Apr 05 2004 Dan Burcaw <dan@ydl.net>
- kernel-g5, kernel-g5-smp packages for
  build_mac32. Obsoleted by ppc64 builds
  of kernel and kernel-smp
- update to 2.6.5 linus

* Sun Mar 28 2004 Dan Burcaw <dan@ydl.net>
- renamed config files for build_mac32
- set defaults for build_mac32 (gcc32)

* Fri Mar 26 2004 Dan Burcaw <dan@ydl.net>
- bug fixes from bk (1.1322)
- spec fixes: fedora merge / build_mac32

* Fri Mar 12 2004 Dan Burcaw <dan@ydl.net>
- 2.6.4 release (1.1319 bk)
- no mkinitrd for now

* Sun Feb 29 2004 Dan Burcaw <dan@ydl.net>
- updated ppc64 configs
- remove devmem patch

* Tue Feb 24 2004 Dan Burcaw <dan@ydl.net>
- Update to 2.6.3-bk5-bk1.1315
